@@ -75,7 +75,7 @@ COMPILER_INTERNALS_OR_DISALLOWED = set([
     ])
 
 
-RE_MANGLE = re.compile('[^\w_]')
+RE_MANGLE = re.compile(r'[^\w_]')
 RE_NAME = re.compile('^%s$' % NAME)
 
 if DEBUG_MODE:
@@ -272,11 +272,11 @@ emit_func_convert_and_escape = template(
 
 class Interpolator(object):
     braces_required_regex = re.compile(
-        r'(?<!\\)\$({(?P<expression>.*)})',
+        r'(\$)?\$({(?P<expression>.*)})',
         re.DOTALL)
 
     braces_optional_regex = re.compile(
-        r'(?<![\\$])\$({(?P<expression>.*)}|(?P<variable>[A-Za-z][A-Za-z0-9_]*))',
+        r'(\$)?\$({(?P<expression>.*)}|(?P<variable>[A-Za-z][A-Za-z0-9_]*))',
         re.DOTALL)
 
     def __init__(self, expression, braces_required, translate=False,
@@ -323,16 +323,26 @@ class Interpolator(object):
             matched = text
             m = self.regex.search(matched)
             if m is None:
+                text = text.replace('$$', '$')
                 nodes.append(ast.Str(s=text))
                 break
 
             part = text[:m.start()]
             text = text[m.start():]
 
+            skip = text.startswith('$$')
+            if skip:
+                part = part + '$'
+
             if part:
-                node = ast.Str(s=part.replace('$$', '$'))
+                part = part.replace('$$', '$')
+                node = ast.Str(s=part)
                 nodes.append(node)
 
+            if skip:
+                text = text[2:]
+                continue
+
             if not body:
                 target = name
             else:
@@ -467,13 +477,6 @@ class ExpressionEngine(object):
 
     >>> eval('not: exists: help')
     False
-
-    >>> eval('string:test ${1}${2}')
-    'test 12'
-
-    >>> eval('string:test $${1}${2}')
-    'test ${1}2'
-
     """
 
     supported_char_escape_set = set(('&', '<', '>'))
@@ -901,6 +904,7 @@ class Compiler(object):
         'translate': Symbol(simple_translate),
         'decode': Builtin("str"),
         'convert': Builtin("str"),
+        'on_error_handler': Builtin("str")
         }
 
     lock = threading.Lock()
@@ -1148,8 +1152,10 @@ class Compiler(object):
         self._leave_assignment((node.name, ))
 
         error_assignment = template(
-            "econtext[key] = cls(__exc, __tokens[__token][1:3])",
+            "econtext[key] = cls(__exc, __tokens[__token][1:3])\n"
+            "if handler is not None: handler(__exc)",
             cls=ErrorInfo,
+            handler=load("on_error_handler"),
             key=ast.Str(s=node.name),
             )
 
